package com.example.addressbook.client;

import com.example.addressbook.client.Contact;
import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasHorizontalAlignment.HorizontalAlignmentConstant;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.smartgwt.client.data.DataSource;
import com.smartgwt.client.data.fields.DataSourceTextField;
import com.smartgwt.client.types.HoverMode;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.Window;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.events.CloseClickEvent;
import com.smartgwt.client.widgets.events.CloseClickHandler;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.fields.HeaderItem;
import com.smartgwt.client.widgets.form.validator.RegExpValidator;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.ListGridRecord;

public class Addressbook implements EntryPoint {
	private static final String SERVER_ERROR = "An error occurred while "
			+ "attempting to contact the server. Please check your network "
			+ "connection and try again.";

	private VerticalPanel mainPanel = new VerticalPanel();
	private HorizontalPanel optionPanel = new HorizontalPanel();
	private ListGrid addressbook = new ListGrid();

	IButton delSelectedButton = new IButton(" Delete selected contacts ");
	IButton delAllButton = new IButton(" Delete all contacts ");
	IButton genSampleContactsButton = new IButton(" Reset contacts ");
	IButton addContactButton = new IButton("Add Contact");
	private AddressbookServiceAsync addressbookSvc = GWT
			.create(AddressbookService.class);

	public void onModuleLoad() {

		showContactGrid();

	}

	public void showContactGrid() {

		addressbook.setWidth(650);
		addressbook.setHeight(300);
		addressbook.setAlternateRecordStyles(true);
		addressbook.setShowAllRecords(true);

		ListGridField firstNameField = new ListGridField("firstName",
				"First Name");
		firstNameField.setCanEdit(true);
		ListGridField lastNameField = new ListGridField("lastName", "Last Name");
		lastNameField.setCanEdit(true);
		ListGridField emailField = new ListGridField("email", "E-mail");
		emailField.setCanEdit(true);
		ListGridField mobileField = new ListGridField("mobile", "Mobile Phone");
		mobileField.setCanEdit(true);

		addressbook.setFields(firstNameField, lastNameField, emailField,
				mobileField);
		addressbook.setCanResizeFields(true);
		addressbook.setData(SampleContacts.getSampleContacts());

		addContactButton.setShowRollOver(true);
		addContactButton.setShowDown(true);
		addContactButton.setWidth(150);

		addContactButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				showAddFormWindow();
			}
		});

		delSelectedButton.setWidth(150);
		delSelectedButton.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				ListGridRecord[] selectedContacts = addressbook
						.getSelectedRecords();
				if (selectedContacts != null)
					for (ListGridRecord rec : selectedContacts)
						addressbook.removeData(rec);
			}
		});

		delAllButton.setWidth(150);
		delAllButton.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				ListGridRecord[] allContacts = addressbook.getRecords();
				for (ListGridRecord rec : allContacts)
					addressbook.removeData(rec);
			}
		});
		genSampleContactsButton.setWidth(150);
		genSampleContactsButton.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				addressbook.setData(SampleContacts.getSampleContacts());
			}
		});

		mainPanel.add(addressbook);
		optionPanel.add(addContactButton);
		optionPanel.add(delAllButton);
		optionPanel.add(delSelectedButton);
		optionPanel.add(genSampleContactsButton);
		optionPanel.setSpacing(5);
		
		mainPanel.add(optionPanel);
		mainPanel.setBorderWidth(5);
		mainPanel.setWidth("650px");
		RootPanel.get("mainContainer").add(mainPanel);

	}

	public void showAddFormWindow() {
		final Window winModal = new Window();

		winModal.setTitle("Add new contact");

		winModal.setShowMinimizeButton(false);
		winModal.setIsModal(true);
		winModal.setShowModalMask(true);
		winModal.centerInPage();
		winModal.setCanDragResize(true);
		winModal.setCanDragReposition(true);
		winModal.setWidth(300);
		winModal.setHeight(250);

		winModal.addCloseClickHandler(new CloseClickHandler() {
			public void onCloseClick(CloseClickEvent event) {
				winModal.destroy();
			}
		});

		DataSource dataSource = new DataSource();
		final DataSourceTextField firstNameField = new DataSourceTextField(
				"firstName", "First Name", 50, true);
		final DataSourceTextField lastNameField = new DataSourceTextField(
				"lastName", "Last Name", 50, true);
		final DataSourceTextField emailField = new DataSourceTextField("email",
				"Email", 100, true);
		RegExpValidator emailValidator = new RegExpValidator();
		emailValidator.setErrorMessage("Invalid email address");
		emailValidator
				.setExpression("^([a-zA-Z0-9_.\\-+])+@(([a-zA-Z0-9\\-])+\\.)+[a-zA-Z0-9]{2,4}$");
		emailField.setValidators(emailValidator);

		final DataSourceTextField mobileField = new DataSourceTextField(
				"mobile", "Mobile phone", 50, false);
		RegExpValidator mobileValidator = new RegExpValidator();
		mobileValidator.setErrorMessage("Invalid mobile phone number");
		mobileValidator
				.setExpression("^(\\(?\\+?[0-9]*\\)?)?[0-9_\\- \\(\\)]*$");
		mobileField.setValidators(mobileValidator);

		dataSource.setFields(firstNameField, lastNameField, emailField,
				mobileField);

		final DynamicForm form = new DynamicForm();
		form.setDataSource(dataSource);
		form.setUseAllDataSourceFields(true);
		HeaderItem header = new HeaderItem();
		header.setDefaultValue("Add Contact Form");

		IButton confirmButton = new IButton();
		confirmButton.setTitle(" Confirm ");

		confirmButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				if (form.validate()) {
					if (form.getValue("mobile") == null)
						form.setValue("mobile", "");

					sendRpcRequest(form);
					winModal.destroy();
				}

			}
		});

		form.setPadding(20);
		form.setFields(header);

		winModal.addItem(form);
		winModal.addItem(confirmButton);

		winModal.show();
	}

	private void sendRpcRequest(DynamicForm form) {
		String[] data = new String[] {
				new String(form.getValue("firstName").toString()),
				new String(form.getValue("lastName").toString()),
				new String(form.getValue("email").toString()),
				new String(form.getValue("mobile").toString()) };

		if (addressbookSvc == null) {
			addressbookSvc = GWT.create(AddressbookService.class);
		}

		// Set up the callback object.
		AsyncCallback<Contact> callback = new AsyncCallback<Contact>() {
			public void onFailure(Throwable caught) {
				// TODO: Do something with errors.
				System.out.println("Adding contact failed");
			}

			public void onSuccess(Contact result) {
				ContactRecord p1 = new ContactRecord();
				p1.setFirstName(result.getFirstName());
				p1.setLastName(result.getLastName());
				p1.setEmail(result.getEmail());
				p1.setMobile(result.getMobile());

				addressbook.addData(p1);
				System.out.println("Adding contact succedded");
			}
		};

		// Make the call to the stock price service.
		addressbookSvc.addContact(data, callback);
	}
}
